#include <oxstd.oxh>
#include <oxdraw.oxh>
#include <oxprob.oxh>
#include <oxfloat.oxh>
#import <maximize>

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
//
// GENERAL ROUTINES FOR ARBITRARI DISTRIBUTIONS AND
// GAS-GPD FILTER
//
// NEW: THE CONFIDENCE BANDS
//		ADD THE BOOTSTRP SE ROUTINE
//
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////
// INITIATE THE CODE: s_iTauMethod::Tau method whether static (=0) or not;
//					  s_dTailProb::Tail region definition--top x% of the dist.
////////////////////////////////////////////////////////////////////////////////////
static decl s_dLambda = 0, s_mData, s_vTau, s_sCountry, s_iIniSpl,
			s_iTauMethod = 4, s_dTailProb = 0.10, s_dSmoothTau, s_vCal;


GetData(const pmData, const piObs, const pvTime, const iLoadSave)
{
	decl mYY, mY, mYDeMean, vTime = <>, i, t, iWithNum, iIniSampleSize, mNewYY, vIndex, vIntraMean;

	if (iLoadSave==0) {
		mYY = deletec(loadmat("Data/ERs_returns.xlsx")[0:][1]'); 
		mNewYY = mYY;
		mY = mYY; mYDeMean = mYY;
	
		// -- in real percentage
		vTime = range(0,99.25,1/252);
		iIniSampleSize = s_iIniSpl-1;
	
	}
	else {
		print("Not specified...");
	}
	
	println("Number of observations = ", columns(mY));
   	println("Number of calendar days = ", columns(vTime));	 

	pmData[0] = mY;
	pvTime[0] = vTime;
	piObs[0] = columns(mY);

}

computeGradient(const func, const vParms, const deltagrad)
{
	decl m, k, mI, mGradient, fbase, vParms2, vParmsTemp;

	print("enter the computerGradient program");
  	m = rows(vParms);
	k = columns(vParms);
	vParmsTemp = vParms;
	if(m==1)
	{
		vParmsTemp = vParms'; 
	}
	m = rows(vParmsTemp);
	mI = unit(m);
	fbase = func(vParmsTemp);
	mGradient = zeros(m,m);
	for(decl i=0;i<m;i++)
	{
		vParms2 = vParmsTemp + deltagrad*mI[][i];
		mGradient[][i] = (func(vParms2)-fbase)/deltagrad ;
	}
	return mGradient;
}

standardErrorsUniv(const objFunction, const restrictionFunction, const vParms, const pvStdErrors)
{
	decl iT, mHess, mJacobian, ir;
	print("enter the univariate standardErrors program");

	iT = rows(vecindex(s_mData.>0));

	ir = Num2Derivative(objFunction, vParms, &mHess);
	print("ir is ", ir);
    if (!ir)
    {	print("Covar() failed in numerical second derivatives\n");
		pvStdErrors[0] = zeros(vParms); 
        return;
    }
    mJacobian = computeGradient(restrictionFunction,vParms, 0.001);
	pvStdErrors[0] = sqrt(diagonal(mJacobian*(invertgen(-mHess,3)/iT)*mJacobian'))';
}


////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
//
// ROUTINES FOR THE GPD SETTING
//
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

fGasGPDFilter(const vData, const omegaD, const omegaX, const alphaD, const alphaX, const betaD, const betaX, const D0, const X0, const dLambda,
						const iReturnFilter=0, const vSMPParms = M_NAN) {
	decl dlik=0, vF, iT = columns(vData);
	decl dXi, dDelta;
	
	vF = X0 | D0;
	decl vOmega = omegaX|omegaD;
	decl mA = diag(alphaX ~ alphaD);
	decl mB = diag(betaX ~ betaD);
	decl vXi, vDelta, vSmoothedScore=0|0;
	if ((iReturnFilter == 1)||(iReturnFilter == 2)) vXi = vDelta = nans(vData);
	decl vPlotXi = nans(vData), vPlotDelta = nans(vData), vPlotDXi = nans(1, columns(vData)-1);
	decl mS = zeros(2, iT);
	decl mTildeS = zeros(2, iT);
	decl mTildeZ = zeros(1, iT+1);
	// run the filter
	for (decl i1=0;i1<iT;++i1) {
		// set parameters
		dXi = exp(vF[0]);
		dDelta = exp(vF[1]);

		if ((iReturnFilter == 1)||(iReturnFilter == 2)) {vXi[i1] = dXi; vDelta[i1] = dDelta;}
		vPlotXi[i1] = dXi;
		vPlotDelta[i1] = dDelta;
				
		/*
		** To BS: Key part of the iteration with smoothing!!!
		*/
		// is there a POT? if so, increase likelihood and update vF	
		vSmoothedScore *= dLambda;
		if (vData[i1] > 0) {
		
			// compute likelihood contribution
			dlik += -log(dDelta) - (1 + 1/dXi)*log(1 + dXi * vData[i1] / dDelta);
			
			// update with GAS step
			mS[][i1] = 
				(
					(1+dXi)*log(1 + dXi * vData[i1] / dDelta)/sqr(dXi)
					+
					(dDelta - (dXi + 3 + 1/dXi) * vData[i1])/(dDelta + dXi * vData[i1])
				)
				|
				(
					sqrt(1+2*dXi) * (vData[i1] - dDelta) / (dDelta + dXi * vData[i1])
				);
			vSmoothedScore += (1 - dLambda)*mS[][i1];

			if (ismissing(dlik) && (iReturnFilter!=1)) break;
		}
		vF = vOmega + mA * vSmoothedScore + mB * vF;
	}

	vPlotDXi = vPlotXi[][1:] - vPlotXi[][:columns(vPlotXi)-2];
	DrawXMatrix(0, vPlotXi, "Xi", s_vCal, 0, 0, 2);
	DrawXMatrix(1, vPlotDelta, "Delta", s_vCal, 0, 0, 2);
	DrawXMatrix(2, -s_vTau, "Tau", s_vCal, 0, 0, 2);
	DrawXMatrix(3, -s_mData, "Data", s_vCal, 0, 0, 2);
	ShowDrawWindow();
	
	if (iReturnFilter==1) return {vDelta,vXi};
	else {
 	if (iReturnFilter==2) {	println("mS[][:5] :", mS[][:5]);
							return {vDelta,vXi,mS}; }
	else return dlik;
	}
}

fTauFilter(const vData, const dTailProbability, const dOmega, const dAlpha, const dBeta) {
// Filter the dynamic tau path using the GAS(1,1) filter, where tau is the upper tail
// quantile with dTailprobability tail mass.
//
// INPUTS:
// vData: 1 x T vector of data
// dTailprobability: double, providing the tail probability for the dynamic quantile
// dOmega, dBeta, dAlpha: three doubles holding the GAS(1,1) parameters for
//			the recursion tau(t+1) = omega + beta tau(t) + alpha score(t)
//
// OUTPUT:
// vTau: 1 x T vector holding the path of filtered quantiles

	decl dTarget = ( 1 - dBeta ) * quantiler(vData, 1 - dTailProbability);

	// start the recursions at the unconditional empirical quantile of all observations
	decl vTau = quantiler(vData, 1 - dTailProbability) * ones(vData);

	// run the filter
	for (decl i1=1;i1<columns(vData);++i1) {
		vTau[i1] = dTarget + dBeta * vTau[i1-1] + dAlpha * ((vData[i1-1] > vTau[i1-1]) -	dTailProbability);
	}
	// return the path of quantiles
	return vTau;
}

fDynamicTau(const vData, const dTailProbability, const dSmoothingParameter=M_NAN, const dMeanReversion=M_NAN) {
// Estimate the GAS(1,1) parameters for a dynamic upper tail
// quantile with dTailprobability tail mass.
//
// INPUTS:
// vData: 1 x T vector of data
// dTailprobability: double, providing the tail probability for the dynamic quantile

// OUTPUT:
// vTau: 1 x T vector holding the path of filtered quantiles
// vPar: P x 1 vector of parameters (output depends on settings in this function)

	// define the quantile objective function to minimize (= max its negative)
	decl fLocalObj = [=](const vP, const adF, const avG, const amH) {

		// check if vP is too weird
		if (fabs(vP[0]) > 0.5 || vP[1] < 0.9 || vP[1] > 1) return 0;
		
		// run the filter for two parameters B and A
		decl vTau;
		vTau = fTauFilter(vData, dTailProbability, 0, vP[0], vP[1]);

		decl vExceedance = vData - vTau;
		adF[0] = double(dTailProbability * meanr(vExceedance) - meanr( vExceedance .* (vExceedance .> 0) ));

		// plot intermediate path of quantiles
		decl vCal = deletec(loadmat("Data/ERs_returns.xlsx")[0:][2]'); 

//		DrawXMatrix(0, vTau, "Tau", vCal, 0, 0, 2);
//		DrawTitle(0, sprint("Fn =", "%8.6f", adF[0], "; Par = ", "%8.6f", vP'));
//		DrawXMatrix(1, vData, "Data", vCal, 0, 0, 2);
//		ShowDrawWindow();
//		CloseDrawWindow();
		
		// return succes or failure indicator
		if (ismissing(adF[0])) return 0;
		return 1;
	};

	// optimize over smoothing parameter
	decl dF = 0;
	decl vP = dSmoothingParameter | dMeanReversion;
	if (isnan(vP)) vP = 0.05 | 0.995;
	MaxControl(-1,1,1);
	MaxControlEps(1e-4, 1e-4);
	decl ir = MaxBFGS(fLocalObj, &vP, &dF, 0, 1);
	decl vTau = fTauFilter(vData, dTailProbability, 0, vP[0], vP[1]);
	
	// return quantiles path and smoothing parameter
	return {vTau, vP[0], vP[1]};
}

fFuncTau(const vData, const dTailProbability, const vParms) {

	// define the quantile objective function to minimize (= max its negative)
	decl dFunc, vP = vParms;
			
		// run the filter for two parameters B and A
		decl vTau = fTauFilter(vData, dTailProbability, 0, vP[0], vP[1]);
		decl vExceedance = vData - vTau;
		dFunc = double(dTailProbability * meanr(vExceedance) - meanr( vExceedance .* (vExceedance .> 0) ));

		return dFunc;
}

fGridSearchParms(const vData, const dGamma, const vParms=M_NAN){

	decl dEvalFunc, vSimA, vSimB, vSimC, mEval, vP, iA, iB, iC, vBestParms;
	
	vSimA = range(-vParms[0]/4, vParms[0]/4, vParms[0]/40) + vParms[0];
	vSimB = range(-(1 - 1e-4 - vParms[1])/2, (1 - 1e-4 - vParms[1])/2, (1 - 1e-4 - vParms[1])/20) + vParms[1];
	
	mEval = <>; 

	for (iA=0; iA<columns(vSimA); ++iA)	{
		print(iA,"/");
		for (iB=0; iB<columns(vSimB); ++iB)	{
				vP = vSimA[iA]|vSimB[iB];
				dEvalFunc = fFuncTau(vData, dGamma, vP);
				if (dEvalFunc!=0) 
					mEval ~= iA|iB|dEvalFunc;
				else
					mEval ~= iA|iB|-10000;
		}
	}

	vBestParms = vSimA[mEval[0][maxcindex(mEval[2][]')]]
				|vSimB[mEval[1][maxcindex(mEval[2][]')]];
	
	return {mEval, vBestParms[0], vBestParms[1]};
}

fTvThresholdMain(const vLoadData) {
	decl nms = {"USDGBP"};
	decl dGamma = 0.10;
	decl dA_tau, dB_tau, dC_tau, vTau, i1, vData, vAtau, vBtau, vY, mX, vBeta, vPlotX, vReturn,
			vParms, dEvalFunc, vSimA, vSimB, mPlot, iA, iB, mEval;

	vAtau = zeros(sizeof(nms), 1);
	vBtau = zeros(sizeof(nms), 1);
	for (i1=0; i1<1; ++i1) {
		vData = vLoadData;
		print(vData[][0:7]);

		// First round of estimating the parameters
		vParms = <0.05; 0.999>;
		[vTau , dA_tau, dB_tau] = fDynamicTau(vData, dGamma, vParms[0], vParms[1]);
  		// Secondly make sure the parameters converged to the global optimal
		vParms = dA_tau|dB_tau;
		[mEval, dA_tau, dB_tau] = fGridSearchParms(vData, dGamma, vParms);
		savemat(sprint("Result/RAWX_Eval_", nms[i1], "_Final.xlsx"), mEval');
		[vTau , dA_tau, dB_tau] = fDynamicTau(vData, dGamma, dA_tau, dB_tau);

		println("Results (NoFilter-DMS) for ", nms[i1], ": Atau = ", double(dA_tau), ": Btau = ", double(dB_tau));
		vAtau[i1][] = dA_tau;
		vBtau[i1][] = dB_tau;		
				
		savemat(sprint("Result/DMS_Tau_", nms[i1], "_Final.xlsx"), vTau');
	}

	savemat("Result/ATau_DMS_Other_Final.xlsx", vAtau);
	savemat("Result/BTau_DMS_Other_Final.xlsx", vBtau);
	
		
}

fComputeTau(const vData, const dTailProbability, const iMethod=4,
	const vScaleOfData=1, const vTrueFilteredQuantiles=0, const dSmoothTau=M_NAN) { 
	// generate thresholds
	decl vTau = nans(vData);
	if (iMethod == 1) {
		// true quantile
		vTau = vScaleOfData .* vTrueFilteredQuantiles;
	} else if (iMethod == 2) {
		// empirical quantile of true prefiltered data * true scale
		vTau = quantiler(vData ./ vScaleOfData, 1-dTailProbability) * vScaleOfData;
	} else if (iMethod == 3) {
		// iGAS time-varying quantile for true prefiltered data * true scale
		vTau = fTauFilter(vData ./ vScaleOfData, dTailProbability, 0, 1, dSmoothTau) .* vScaleOfData;
	} else if (iMethod == 4) {
		// iGAS time-varying quantile for unfiltered data
		vTau = loadmat(sprint("Result/DMS_Tau_",s_sCountry,"_Final.xlsx"))';		
	} else {
		// (static) empirical quantile of unfiltered data
		vTau = constant(quantiler(vData, 1-dTailProbability), vData);
	}
	s_vTau = vTau;
	return vTau;
}



////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
//
// ROUTINES FOR THE STUDENT T SETTING
//
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

restrictParms(const vParms)
{
	decl vRestrictedParms = vParms;
 	decl iMode = 1;

	vRestrictedParms[0:1] = exp(vParms[0:1]);
	vRestrictedParms[2:3] = 1 ./ (1 + exp(-vParms[2:3]));
	vRestrictedParms[4:] = vParms[4:];
	
	return vRestrictedParms;
}


fStaticLikelihood(const vData, const dDelta, const dXi) {
// The average log-likelihood of a static GPD
//
// Return: the average log likelihood for the data
	return meanr( -log(dDelta) - (1 + 1/dXi) * log(1 + dXi * vData ./ dDelta) );
}

fStaticEstimate(const vData) {

		decl iReturn, dF, vP = log(<1;1>), vSaveMSG; 

		MaxControl(-1,0,1);
		iReturn =
			MaxBFGS(
				[=](const vP, const adF, const avG, const amH) {
					adF[0] = fStaticLikelihood(vData, exp(vP[1]), exp(vP[0]));
					if (ismissing(adF[0])) return 0; else return 1;
				},
				&vP, &dF, 0, 1
			);
		vSaveMSG = iReturn | dF;

	return { exp(vP[0]) , exp(vP[1]) };

}

fGasGPDFilterlik(const vP, const adF, const avG, const amH)
{																
	decl vData = s_mData; 
	decl dTail, vS, dGridSize, iTauComputationMethod, vTrueQuantiles, vXi, vDelta, vNu;
	dTail = s_dTailProb; 
	dGridSize = 10000;
	iTauComputationMethod = s_iTauMethod;
	
	vXi = zeros(vData);
	vDelta = zeros(vData);

	// compute starting values
	decl D0, X0, vpStdErr, vLoadData, vIniData, dThr;

	decl vTau = fComputeTau(vData, dTail, iTauComputationMethod, 1, 0, s_dSmoothTau);
	s_vTau = vTau;
		
	vLoadData = vData - vTau;
	vIniData = vLoadData[][vecindex(vData.>vTau)];
	
	decl vParms = restrictParms(vP);
	adF[0] = fGasGPDFilter(vData - vTau,
				vParms[5]*(1-vParms[3]), vParms[4]*(1-vParms[2]), vParms[1], vParms[0], vParms[3], vParms[2], vParms[5], vParms[4], s_dLambda, 0, zeros(2,1)) / sumr(vData.>vTau)
				- ((vData - vTau)*(dTail -
				(vData .<= vTau))') / sumr(vData.<=vTau);
	if (ismissing(adF[0])) return 0; else return 1;

}

fLogLikelihood(const vP, const vData) {
// the core likelihood function
// you can add more arguments if wanted
// IMPORTANT: it returns a 1 x T vector of likelihood
//     contributions

	decl dlik=0, vF, iT = columns(vData);
	decl vlik = zeros(1, iT);
	decl X0, D0;
	decl dXi, dDelta;
	decl vParms = restrictParms(vP);
	X0 = vParms[4];
	D0 = vParms[5];
	vF = X0 | D0;
	decl mA = diag(vParms[0]~vParms[1]);
	decl mB = diag(vParms[2]~vParms[3]);
	decl vOmega = (unit(2) - mB)*vParms[4:5];
	decl dLambda = s_dLambda;		
	decl vXi, vDelta, vSmoothedScore=0|0;
	vXi = vDelta = nans(vData);
	decl mS = zeros(2, iT);
	// run the filter
	for (decl i1=0;i1<iT;++i1) {
		// set parameters
		dXi = exp(vF[0]);
		dDelta = exp(vF[1]);
		vXi[i1] = dXi; vDelta[i1] = dDelta;		

		// is there a POT? if so, increase likelihood and update vF
		vSmoothedScore *= dLambda;
		if (vData[i1] > 0) {
		
			// compute likelihood contribution
			vlik[i1] = -log(dDelta) - (1 + 1/dXi)*log(1 + dXi * vData[i1] / dDelta);
			
			// update with GAS step
			mS[][i1] = 
				(
					(1+dXi)*log(1 + dXi * vData[i1] / dDelta)/sqr(dXi)
					+
					(dDelta - (dXi + 3 + 1/dXi) * vData[i1])/(dDelta + dXi * vData[i1])
				)
				|
				(
					sqrt(1+2*dXi) * (vData[i1] - dDelta) / (dDelta + dXi * vData[i1])
				);
			vSmoothedScore += (1 - dLambda)*mS[][i1];
			if (ismissing(vlik[i1])) break;
		} 
		vF = vOmega + mA * vSmoothedScore + mB * vF;
	}

	return vlik;
}


fInfoCriterion(const n, const k, const loglik) // n::sample size; k::parameter number; loglik: converged loglik
{
	decl AIC, AICc, BIC;

	AIC = 2*k - 2*loglik;
	AICc = AIC + (2*k*k+2*k)/(n-k-1); // Modification for small sample size, not really used for this study
	BIC = log(n)*k - 2*loglik;

	return {AIC, AICc, BIC};
}


fDirectEstimation(const dTail, const dGridSize,
	const iTauComputationMethod, const dTauSmoothingParameter=M_NAN) {

	// generate data

	decl vData = s_mData;
	decl iT = columns(vData);

	decl vP = <-3.5656;-3.5183;3.75;3.75;-0.05;-0.05>, dF=0;

	if (s_sCountry=="USDGBP") vP = <-4.8130;-2.4536;6.2369;4.8020;-2.0186;-1.1842>;
	
	// compute starting values
	decl D0, X0, vpStdErr;
	decl vDeltaFit, vXiFit, vDeltaFitH1, vXiFitH1, mS, mGvec, mCI, mVar, amC = 0, vpStdErrSDW, vNewP;
	
	println(s_sCountry~" Tau smooth parm = ", s_dSmoothTau);
	MaxControl(2500,100,0);
	decl iReturn =
		MaxBFGS(fGasGPDFilterlik,
			&vP, &dF, 0, 1
		);
	if (iReturn != MAX_CONV) println("SIMULATION: ",MaxConvergenceMsg(iReturn));
	CloseDrawWindow();
	
	decl AIC, AICc, BIC, dFF;
	dFF = dF;
	[AIC, AICc, BIC] = fInfoCriterion(columns(s_mData), rows(vP), dF*columns(s_mData));

	standardErrorsUniv(fGasGPDFilterlik, restrictParms, vP, &vpStdErr);
	
	print("\n", iReturn,
		 "\n", MaxConvergenceMsg(iReturn),
        "\nFunction value = ", "%f", dFF*columns(s_mData), "; \n",
        "\n<AIC, BIC> = <", "%f", AIC, "%f", BIC, ">; \n",
		 "======== "~s_sCountry~" Results: Parameters========\n",
  		 "Optimization results in ", 
         "%r", {},
         "%c", {"value", "Calculated", "stderr", "P-value", "Lik", "AIC", "AICc", "BIC"},
		 vP~restrictParms(vP)~vpStdErr~2*tailn(fabs( restrictParms(vP)./vpStdErr ))
		 ~dFF*columns(s_mData)*ones(vP)~AIC*ones(vP)~AICc*ones(vP)~BIC*ones(vP));					
	
}




////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
//
// ROUTINES FOR SCENARIOS ON TIME VARYING PARAMETERS AND MAIN()
//
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////

EstimationMain(const GPDorSTUDENT, const iN, const iT, const CompRisk, const PlotSignal, const vLoadData = M_NAN) {

	// magic constants
	decl dTail = s_dTailProb; //0.05
	decl dGridSize = 10000;
	decl dSmoothTau = 0.1;
	if (s_sCountry=="USDGBP") dSmoothTau = loadmat("Result/ATau_DMS_Other_Final.xlsx")[0][0];
	if (s_sCountry=="Brent") dSmoothTau = loadmat("Result/ATau_DMS_Other_Final.xlsx")[1][0];
	
	s_dSmoothTau = dSmoothTau;
	decl TauComputationMethod = s_iTauMethod;		// 0, or else: (static) empiric8al quantile of unfiltered data
										// 1: true quantile
										// 2: empirical quantile of true prefiltered data * true scale
										// 3: iGAS(dSmoothTau) time-varying quantile for true prefiltered data * true scale
										// 4: iGAS(dSmoothTau) time-varying quantile for unfiltered data

	// generate pattern for Xi and Delta
	decl vX, vXi, vDelta, vXCal, vXiCal, vDeltaCal;
	decl a,	vVaR, vES, vVolParm, vEffect;
	
	// simulate data and filter back for one simulation
	decl vDeltaFit, vXiFit, vData, vP, mC, i, vTrueQuantiles;
	for (i=0;i<iN;i++) 
	{
		ranseed(20200522+i);
		s_mData = vLoadData;
		fDirectEstimation(dTail, dGridSize, TauComputationMethod, s_dSmoothTau); 
		
	}
	
}


main()
{
	decl GPDorSTUDENT, iN, iT, vsNames, u, i, sDataNAME, mData, vTime, vUseData, vParm, mConvMSG,
			mAid1, mAid2 = <>, mParms, mInParms, vSelVec, mLoadParms;

	GPDorSTUDENT = 1;
	iN = 1;	

	s_iIniSpl = 2500;  // Initial sample of 3 months
	vsNames = {"USDGBP"};	   // "ONE" has been under 0.5 for the convergence
	
	for (u=0; u<1; u++) {

		s_sCountry = vsNames[u];
		sDataNAME =	{s_sCountry~" return"};

		println("\n===== Getting "~sprint(s_sCountry)~" data... =====");
		GetData(&mData, &iT, &vTime, 0);
		vUseData = -100*mData;
		s_vCal = deletec(loadmat("Data/ERs_Returns.xlsx")[0:][2]');

		fTvThresholdMain(vUseData);
		EstimationMain(GPDorSTUDENT, iN, iT, "TRUE", "FALSE", vUseData);
	}
		 
}

